from sklearn.datasets import make_circles
import numpy as np
import matplotlib.pyplot as plt
from sklearn.svm import LinearSVC,SVC
from sklearn.preprocessing import StandardScaler
from mpl_toolkits import mplot3d
import seaborn as sns


#Robimy sobie zbiór danych liniowo nieseparowalny
X,y = make_circles(n_samples=400, shuffle=True, noise=0.15, random_state=12, factor=0.03)
reds = y==0
blues = y==1
plt.scatter(X[reds,0],X[reds,1],c='r')
plt.scatter(X[blues,0],X[blues,1],c='b')

#Mimo to niestrudzenie używamy liniowego SVM, żeby wykonać klasyfikację

svc = LinearSVC(loss='hinge', C=1)
svc.fit(X, y)
xt = np.linspace(-2,2,10)
yt = (-svc.intercept_-svc.coef_[0][0]*xt) / svc.coef_[0][1]
plt.plot(xt,yt)
plt.show()

# Transformujemy z D2->3D
Ztr = np.column_stack((np.array(X[:,0]), np.array(X[:,1]), np.array(X[:,0]**2+X[:,1]**2)))

# Dokładnie ten sam model
svc = LinearSVC(loss='hinge', C=1)
svc.fit(Ztr, y)
z = lambda x,y: (-svc.intercept_[0]-svc.coef_[0][0]*x-svc.coef_[0][1]*y) / svc.coef_[0][2]
tmp = np.linspace(-2,2,10)
xt,yt = np.meshgrid(tmp,tmp)


# Rysujemy wszyscko w 3D
ax = plt.axes(projection='3d')
ax.scatter(Ztr[reds,0],Ztr[reds,1],Ztr[reds,2],c='r')
ax.scatter(Ztr[blues,0],Ztr[blues,1],Ztr[blues,2],c='b')
ax.plot_surface(xt, yt, z(xt,yt), alpha=0.5)

plt.show()
